//Settings
const float vacMaxTime = 5; //Maximum vacuum time in minutes
const float heatMaxTime = 5; //Maximum heating time in minutes
const bool enableBuzzer = true; //True enables the buzzer, false disables it

//Outputs
const int heater = 2;
const int vacuum = 3;
const int buzzer = 4;

//Inputs
const int vacLimitSwitch = 5;
const int heatLimitSwitch = 6;
const int heaterTimeDial = A0;
const int vacuumTimeDial = A1;

//Properties
bool vacCicleDone = false; //Flag to indicate if the vacuum has been activated while the limit switch is still active 
bool heatCicleDone = false; //Flag to indicate if the heater has been activated while the limit switch is still active 
int vacLimitSwitchState = 0; //Variable to store the vacuum limit switch state
int heatLimitSwitchState = 0; //Variable to store the heater limit switch state
const int powerOn = 0; //Flag for Power ON - to improve code readability
const int powerOff = 1; //Flag for Power OFF - to improve code readability
long int vacuumTime = 0; //Set vacuum time in millis
long int heaterTime = 0; //Set heating time in millis
const float vacTimeFactor = (vacMaxTime*60*1000)/1024; //Constant used to calculate the set vacuum time based on the max time config and the potentiometer reading.
const float heatTimeFactor = (heatMaxTime*60*1000)/1024; //Constant used to calculate the set heating time based on the max time config and the potentiometer reading.
unsigned long previousVacMillis = 0; //Variable to store the vacuum timer start in millis
unsigned long previousHeatMillis = 0; //Variable to store the heating timer start in millis
const unsigned int freq = 500; //Passive buzzer frequency in Hz


//Main Code
void setup() {
  Serial.begin(9600);
  pinMode(vacLimitSwitch, INPUT_PULLUP);
  pinMode(heatLimitSwitch, INPUT_PULLUP);
  pinMode(vacuumTimeDial, INPUT);
  pinMode(heaterTimeDial, INPUT);
  pinMode(heater, OUTPUT);
  pinMode(vacuum, OUTPUT);
  pinMode(buzzer, OUTPUT);
  digitalWrite(heater, HIGH); //Default relay state (HIGH = OFF, LOW = ON);
  digitalWrite(vacuum, HIGH); //Default relay state (HIGH = OFF, LOW = ON);
}

void loop() {

  unsigned long currentMillis = millis();
  unsigned long elapsedHeatTime = currentMillis - previousHeatMillis;
  unsigned long elapsedVacTime = currentMillis - previousVacMillis;
  
  vacLimitSwitchState = digitalRead(vacLimitSwitch);
  heatLimitSwitchState = digitalRead(heatLimitSwitch);
  vacuumTime = analogRead(vacuumTimeDial) * vacTimeFactor;
  heaterTime = analogRead(heaterTimeDial) * heatTimeFactor;
  
  if(analogRead(heaterTimeDial) < 60){
    //Heater timer is OFF -> Turn heater OFF
    turnDevicePower(powerOff,heater,"heater");
    heatCicleDone = false;
  } else if(heatLimitSwitchState == LOW && digitalRead(heater) == HIGH && !heatCicleDone){
    //Limit switch was hit -> Start the heater and the timer
    if(enableBuzzer){
      //Makes a single beep to alert that the heater has been turned on
      singleBeep();
      }
     previousHeatMillis = millis();
     turnDevicePower(powerOn,heater,"heater");
     Serial.println("For "+getRealTime(heaterTime));
  } else if(elapsedHeatTime > heaterTime && digitalRead(heater) == LOW && !heatCicleDone ) {
    //Turns the heater off since the set time has elapsed
      if(enableBuzzer){
        //Makes a double-beep to alert that the heating is done
        doubleBeep();
      }
      turnDevicePower(powerOff,heater,"heater");
      heatCicleDone = true;
      delay(500);
  } else if(heatLimitSwitchState == HIGH && heatCicleDone){
     //Resets the flag, letting the user vacuum form again
     heatCicleDone = false;
  }

  if(analogRead(vacuumTimeDial) < 60){
    //Vacuum timer is OFF -> Turn vacuum OFF
    turnDevicePower(powerOff,vacuum,"vacuum");
    vacCicleDone = false;
  } else if(vacLimitSwitchState == LOW && digitalRead(vacuum) == HIGH && !vacCicleDone) {
    //Limit switch was hit -> Start the vacuum and the timer
    previousVacMillis = millis();
    turnDevicePower(powerOn,vacuum,"vacuum");
    Serial.println("For "+getRealTime(vacuumTime));
  } else if(elapsedVacTime > vacuumTime && digitalRead(vacuum) == LOW && !vacCicleDone ){
     //Turns the vacuum off since the set time has elapsed
     turnDevicePower(powerOff,vacuum,"vacuum");
     vacCicleDone = true;
     delay(500);
  } else if(vacLimitSwitchState == HIGH && vacCicleDone){
     //Resets the flag, letting the user vacuum form again
     vacCicleDone = false;
  }
}
